<?php
/**
 * Copyright (C) 2023  Jaap Jansma (jaap.jansma@civicoop.org)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

namespace Drupal\cmrf_form_processor;

use Drupal\cmrf_core\Core;
use Drupal\cmrf_form_processor\Plugin\WebformHandler\OptionsSet;

class Factory {

  /**
   * The CiviMRF Core.
   *
   * @var \Drupal\cmrf_core\Core
   */
  protected $core;

  protected $defaultCache = '1 hour';

  public function __construct(Core $core) {
    $this->core = $core;
  }

  public function rebuildOptionSets() {
    foreach($this->core->getConnectors() as $connector => $connectorTitle) {
      foreach($this->formProcessorList($connector) as $formProcessor => $formProcessorLabel)
        $this->formProcessorFields($connector, $formProcessor);
    }
  }

  public function getConnectors() {
    return $this->core->getConnectors();
  }

  public function getDefaultCache() {
    return $this->defaultCache;
  }

  /**
   * Get the list of form processors for the given connection.
   *
   * @param string $connection
   *   The connection for which to retrieve the form processors.
   *
   * @return array
   *   The list of form processors.
   */
  public function formProcessorList($connection) {
    $call = $this->core->createCall($connection, 'FormProcessorInstance', 'list', [], ['limit' => 0, 'cache' => $this->defaultCache]);
    $reply = $this->core->executeCall($call);
    $values = [];
    if (is_array($reply) && isset($reply['values'])) {
      $values = $reply['values'];
    } elseif (is_object($reply)) {
      $values = $reply->getValues();
    }
    return array_map(function ($value) {
      return $value['title'];
    }, $values);
  }

  /**
   * Get the fields for the form processor.
   *
   * @param string $connection
   *   The connection for the form processor.
   * @param string $formprocessor
   *   The form processor.
   *
   * @return array
   *   The fields for the form processor.
   */
  public function formProcessorFields($connection, $formprocessor) {
    $connections = $this->core->getConnectors();
    $call = $this->core->createCall($connection, 'FormProcessor', 'getfields', ['api_action' => $formprocessor], ['limit' => 0, 'cache' => $this->defaultCache]);
    $reply = $this->core->executeCall($call);
    $fields = [];
    if (is_array($reply) && isset($reply['values'])) {
      $fields = $reply['values'];
    } elseif (is_object($reply)) {
      $fields = $reply->getValues();
    }
    $formProcessors = $this->formProcessorList($connection);
    return $fields;
  }

}
